import json
import uuid
from typing import Tuple

from typing_extensions import deprecated

from pycti.utils.opencti_stix2_identifier import (
    external_reference_generate_id,
    kill_chain_phase_generate_id,
)
from pycti.utils.opencti_stix2_utils import (
    STIX_CYBER_OBSERVABLE_MAPPING,
    SUPPORTED_INTERNAL_OBJECTS,
    SUPPORTED_STIX_ENTITY_OBJECTS,
)

OPENCTI_EXTENSION = "extension-definition--ea279b3e-5c71-4632-ac08-831c66a786ba"

supported_types = (
    SUPPORTED_STIX_ENTITY_OBJECTS  # entities
    + SUPPORTED_INTERNAL_OBJECTS  # internals
    + list(STIX_CYBER_OBSERVABLE_MAPPING.keys())  # observables
    + ["relationship", "sighting"]  # relationships
    + ["pir"]
)


def is_id_supported(key):
    if "--" in key:
        id_type = key.split("--")[0]
        return id_type in supported_types
    # If not a stix id, don't try to filter
    return True


class OpenCTIStix2Splitter:
    def __init__(self):
        self.cache_index = {}
        self.cache_refs = {}
        self.elements = []
        self.incompatible_items = []

    def get_internal_ids_in_extension(self, item):
        ids = []
        if item.get("x_opencti_id"):
            ids.append(item["x_opencti_id"])
        if (
            item.get("extensions")
            and item["extensions"].get(OPENCTI_EXTENSION)
            and item["extensions"].get(OPENCTI_EXTENSION).get("id")
        ):
            ids.append(item["extensions"][OPENCTI_EXTENSION]["id"])
        return ids

    def enlist_element(
        self, item_id, raw_data, cleanup_inconsistent_bundle, parent_acc
    ):
        nb_deps = 1
        if item_id not in raw_data:
            return 0

        existing_item = self.cache_index.get(item_id)
        if existing_item is not None:
            return existing_item["nb_deps"]

        item = raw_data[item_id]
        if self.cache_refs.get(item_id) is None:
            self.cache_refs[item_id] = []
        for key in list(item.keys()):
            value = item[key]
            # Recursive enlist for every refs
            if key.endswith("_refs") and item[key] is not None:
                to_keep = []
                for element_ref in item[key]:
                    # We need to check if this ref is not already a reference
                    is_missing_ref = raw_data.get(element_ref) is None
                    must_be_cleaned = is_missing_ref and cleanup_inconsistent_bundle
                    not_dependency_ref = (
                        self.cache_refs.get(element_ref) is None
                        or item_id not in self.cache_refs[element_ref]
                    )
                    # Prevent any self reference
                    if (
                        is_id_supported(element_ref)
                        and not must_be_cleaned
                        and element_ref not in parent_acc
                        and element_ref != item_id
                        and not_dependency_ref
                    ):
                        self.cache_refs[item_id].append(element_ref)
                        nb_deps += self.enlist_element(
                            element_ref,
                            raw_data,
                            cleanup_inconsistent_bundle,
                            parent_acc + [element_ref],
                        )
                        if element_ref not in to_keep:
                            to_keep.append(element_ref)
                    item[key] = to_keep
            elif key.endswith("_ref"):
                is_missing_ref = raw_data.get(value) is None
                must_be_cleaned = is_missing_ref and cleanup_inconsistent_bundle
                not_dependency_ref = (
                    self.cache_refs.get(value) is None
                    or item_id not in self.cache_refs[value]
                )
                # Prevent any self reference
                if (
                    value is not None
                    and not must_be_cleaned
                    and value not in parent_acc
                    and is_id_supported(value)
                    and value != item_id
                    and not_dependency_ref
                ):
                    self.cache_refs[item_id].append(value)
                    nb_deps += self.enlist_element(
                        value,
                        raw_data,
                        cleanup_inconsistent_bundle,
                        parent_acc + [value],
                    )
                else:
                    item[key] = None
            # Case for embedded elements (deduplicating and cleanup)
            elif key == "external_references" and item[key] is not None:
                # specific case of splitting external references
                # reference_ids = []
                deduplicated_references = []
                deduplicated_references_cache = {}
                references = item[key]
                for reference in references:
                    reference_id = external_reference_generate_id(
                        url=reference.get("url"),
                        source_name=reference.get("source_name"),
                        external_id=reference.get("external_id"),
                    )
                    if (
                        reference_id is not None
                        and deduplicated_references_cache.get(reference_id) is None
                    ):
                        deduplicated_references_cache[reference_id] = reference_id
                        deduplicated_references.append(reference)
                        # - Needed for a future move of splitting the elements
                        # reference["id"] = reference_id
                        # reference["type"] = "External-Reference"
                        # raw_data[reference_id] = reference
                        # if reference_id not in reference_ids:
                        #     reference_ids.append(reference_id)
                        # nb_deps += self.enlist_element(reference_id, raw_data)
                item[key] = deduplicated_references
            elif key == "kill_chain_phases" and item[key] is not None:
                # specific case of splitting kill_chain phases
                # kill_chain_ids = []
                deduplicated_kill_chain = []
                deduplicated_kill_chain_cache = {}
                kill_chains = item[key]
                for kill_chain in kill_chains:
                    kill_chain_id = kill_chain_phase_generate_id(
                        kill_chain_name=kill_chain.get("kill_chain_name"),
                        phase_name=kill_chain.get("phase_name"),
                    )
                    if (
                        kill_chain_id is not None
                        and deduplicated_kill_chain_cache.get(kill_chain_id) is None
                    ):
                        deduplicated_kill_chain_cache[kill_chain_id] = kill_chain_id
                        deduplicated_kill_chain.append(kill_chain)
                        # - Needed for a future move of splitting the elements
                        # kill_chain["id"] = kill_chain_id
                        # kill_chain["type"] = "Kill-Chain-Phase"
                        # raw_data[kill_chain_id] = kill_chain
                        # if kill_chain_id not in kill_chain_ids:
                        #     kill_chain_ids.append(kill_chain_id)
                        # nb_deps += self.enlist_element(kill_chain_id, raw_data)
                item[key] = deduplicated_kill_chain

        # Get the final dep counting and add in cache
        item["nb_deps"] = nb_deps
        # Put in cache
        if self.cache_index.get(item_id) is None:
            # enlist only if compatible
            if item["type"] == "relationship":
                is_compatible = (
                    item["source_ref"] is not None and item["target_ref"] is not None
                )
            elif item["type"] == "sighting":
                is_compatible = (
                    item["sighting_of_ref"] is not None
                    and len(item["where_sighted_refs"]) > 0
                )
            else:
                is_compatible = is_id_supported(item_id)
            if is_compatible:
                self.elements.append(item)
            else:
                self.incompatible_items.append(item)
            self.cache_index[item_id] = item
            for internal_id in self.get_internal_ids_in_extension(item):
                self.cache_index[internal_id] = item

        return nb_deps

    def split_bundle_with_expectations(
        self,
        bundle,
        use_json=True,
        event_version=None,
        cleanup_inconsistent_bundle=False,
    ) -> Tuple[int, list, list]:
        """splits a valid stix2 bundle into a list of bundles"""
        if use_json:
            try:
                bundle_data = json.loads(bundle)
            except:
                raise Exception("File data is not a valid JSON")
        else:
            bundle_data = bundle

        if "objects" not in bundle_data:
            raise Exception("File data is not a valid bundle")
        if "id" not in bundle_data:
            bundle_data["id"] = "bundle--" + str(uuid.uuid4())

        raw_data = {}

        # Build flat list of elements
        for item in bundle_data["objects"]:
            raw_data[item["id"]] = item
            for internal_id in self.get_internal_ids_in_extension(item):
                raw_data[internal_id] = item
        for item in bundle_data["objects"]:
            self.enlist_element(item["id"], raw_data, cleanup_inconsistent_bundle, [])

        # Build the bundles
        bundles = []

        def by_dep_size(elem):
            return elem["nb_deps"]

        self.elements.sort(key=by_dep_size)

        elements_with_deps = list(
            map(lambda e: {"nb_deps": e["nb_deps"], "elements": [e]}, self.elements)
        )

        number_expectations = 0
        for entity in elements_with_deps:
            number_expectations += len(entity["elements"])
            bundles.append(
                self.stix2_create_bundle(
                    bundle_data["id"],
                    entity["nb_deps"],
                    entity["elements"],
                    use_json,
                    event_version,
                )
            )

        return number_expectations, self.incompatible_items, bundles

    @deprecated("Use split_bundle_with_expectations instead")
    def split_bundle(self, bundle, use_json=True, event_version=None) -> list:
        _, _, bundles = self.split_bundle_with_expectations(
            bundle, use_json, event_version
        )
        return bundles

    @staticmethod
    def stix2_create_bundle(bundle_id, bundle_seq, items, use_json, event_version=None):
        """create a stix2 bundle with items

        :param items: valid stix2 items
        :type items:
        :param use_json: use JSON?
        :type use_json:
        :return: JSON of the stix2 bundle
        :rtype:
        """

        bundle = {
            "type": "bundle",
            "id": bundle_id,
            "spec_version": "2.1",
            "x_opencti_seq": bundle_seq,
            "objects": items,
        }
        if event_version is not None:
            bundle["x_opencti_event_version"] = event_version
        return json.dumps(bundle) if use_json else bundle



class OpenCTIApiPir:
    """OpenCTIApiPir"""

    def __init__(self, api):
        self.api = api

    def pir_flag_element(self, **kwargs):
        id = kwargs.get("id", None)
        input = kwargs.get("input", None)
        query = """
            mutation PirFlagElement($id: ID!, $input: PirFlagElementInput!) {
                pirFlagElement(id: $id, input: $input)
            }
           """
        self.api.query(
            query,
            {
                "id": id,
                "input": input,
            },
        )

    def pir_unflag_element(self, **kwargs):
        id = kwargs.get("id", None)
        input = kwargs.get("input", None)
        query = """
            mutation PirUnflagElement($id: ID!, $input: PirUnflagElementInput!) {
                pirUnflagElement(id: $id, input: $input)
            }
           """
        self.api.query(
            query,
            {
                "id": id,
                "input": input,
            },
        )



import secrets
from typing import Dict, List, Optional


class User:
    """Representation of a user on the OpenCTI platform

    Users can be member of multiple groups, from which its permissions
    (capabilities) are derived. Additionally, users are part of organisations,
    and sometimes administrating them (Enterprise edition).

    They have configured confidence, and an effective confidence (which might
    be set by the group).

    You can view the properties, token_properties, session_properties, and
    me_properties attributes of a User object to view what attributes will be
    present in a User or MeUser object.
    """

    def __init__(self, opencti):
        self.opencti = opencti
        self.properties = """
            id
            standard_id
            individual_id
            user_email
            firstname
            lastname
            name
            description
            language
            theme
            unit_system
            external
            restrict_delete
            account_status
            account_lock_after_date
            entity_type
            parent_types
            created_at
            updated_at
            unit_system
            submenu_show_icons
            submenu_auto_collapse
            monochrome_labels
            roles {
                id, name, description
                capabilities {
                    id, name
                }
            }
            groups {
                edges {
                    node {
                        id, name, description
                    }
                }
            }
            objectOrganization {
                edges {
                    node {
                        id, is_inferred, name, description
                    }
                }
            }
            administrated_organizations {
                id, name, description
            }
            user_confidence_level {
                max_confidence
                overrides {
                    entity_type, max_confidence
                }
            }
            effective_confidence_level {
                max_confidence
                source {
                    type
                    object {
                        ... on Group {
                            id, name
                        }
                    }
                }
                overrides {
                    entity_type, max_confidence
                    source {
                        type
                        object {
                            ... on Group {
                                id, name
                            }
                        }
                    }
                }
            }
        """

        self.token_properties = """
            api_token
        """

        self.session_properties = """
            sessions {
                id, created, ttl, originalMaxAge
            }
        """

        self.me_properties = """
            id
            individual_id
            user_email
            firstname
            lastname
            name
            description
            theme
            language
            unit_system
            submenu_show_icons
            submenu_auto_collapse
            entity_type
            parent_types
            created_at
            updated_at
            objectOrganization {
                edges {
                    node {
                        id, name
                    }
                }
            }
            administrated_organizations {
                id, name
            }
            capabilities {
                id, name, description
            }
            groups {
                edges {
                    node {
                        id, name, description
                    }
                }
            }
            effective_confidence_level {
                max_confidence
                source {
                    type
                    object {
                        ... on Group {
                            id, name
                        }
                    }
                }
                overrides {
                    entity_type, max_confidence
                    source {
                        type
                        object {
                            ... on Group {
                                id, name
                            }
                        }
                    }
                }
            }
        """

    def list(self, **kwargs) -> List[Dict]:
        """Search/list users on the platform

        Searches users given some conditions. Defaults to listing all users.

        :param first: Defaults to 500. Retrieve this number of results.
        :type first: int, optional
        :param after: Retrieves all results after the user with this ID.
            Ignored if None, empty, or if fetching all results, defaults to
            None.
        :type after: str, optional
        :param orderBy:  Orders results by this field.
            Can be one of user, user_email, firstname, lastname, language,
            external, created_at, updated_at, or _score, defaults to "name".
        :type orderBy: str, optional
        :param orderMode:  Ordering direction. Must be one
            of "asc" or "desc", defaults to "asc".
        :type orderMode: str, optional
        :param filters:  OpenCTI API FilterGroup object.
            This is an advanced parameter. To learn more please search for
            the FilterGroup object in the OpenCTI GraphQL Playground, defaults
            to {}.
        :type filters: dict, optional
        :param search:  String to search for when listing
            users, defaults to None.
        :type search: str, optional
        :param include_sessions:  Whether or not to
            include a list of sessions with results, defaults to False.
        :type include_sessions: bool, optional
        :param customAttributes: Custom attributes to fetch from the GraphQL
            query
        :type customAttributes: str, optional
        :param getAll: Defaults to False. Whether or not to get all results
            from the search. If True then param first is ignored.
        :type getAll: bool, optional
        :param withPagination: Defaults to False. Whether to return pagination
            info with results.
        :type withPagination: bool, optional
        :return: Returns a list of users, sorted as specified.
        :rtype: list[dict]
        """
        first = kwargs.get("first", 500)
        after = kwargs.get("after", None)
        order_by = kwargs.get("orderBy", "name")
        order_mode = kwargs.get("orderMode", "asc")
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        include_sessions = kwargs.get("include_sessions", False)
        custom_attributes = kwargs.get("customAttributes", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("withPagination", False)

        if get_all:
            first = 100

        self.opencti.admin_logger.info(
            "Fetching users with filters", {"filters": filters}
        )
        query = (
            """
            query UserList($first: Int, $after: ID, $orderBy: UsersOrdering, $orderMode: OrderingMode, $filters: FilterGroup, $search: String) {
                users(first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode, filters: $filters, search: $search) {
                    edges {
                        node {
                    """
            + (self.properties if custom_attributes is None else custom_attributes)
            + (self.session_properties if include_sessions else "")
            + """
                        }
                    }

                    pageInfo {
                        startCursor, endCursor, hasNextPage, hasPreviousPage
                        globalCount
                    }
                }
            }
            """
        )
        result = self.opencti.query(
            query,
            {
                "first": first,
                "after": after,
                "orderBy": order_by,
                "orderMode": order_mode,
                "filters": filters,
                "search": search,
            },
        )

        if get_all:
            final_data = []
            data = self.opencti.process_multiple(result["data"]["users"])
            final_data = final_data + data
            while result["data"]["users"]["pageInfo"]["hasNextPage"]:
                after = result["data"]["users"]["pageInfo"]["endCursor"]
                result = self.opencti.query(
                    query,
                    {
                        "first": first,
                        "after": after,
                        "orderBy": order_by,
                        "orderMode": order_mode,
                        "filters": filters,
                        "search": search,
                    },
                )
                data = self.opencti.process_multiple(result["data"]["users"])
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["users"], with_pagination
            )

    def read(self, **kwargs) -> Optional[Dict]:
        """Reads user details from the platform.

        :param id: ID of the user to fetch
        :type id: str, optional
        :param include_sessions:  Whether or not to
            include a list of sessions for the given user, defaults to False.
        :type include_sessions: bool, optional
        :param include_token:  Whether or not to include
            the user's API token, defaults to False.
        :type include_token: bool, optional
        :param customAttributes: Custom attributes to include instead of the
            defaults
        :type customAttribues: str, optional
        :param filters: Filters to apply to find a single user
        :type filters: dict, optional
        :param search: Search term to use to find a single user
        :type search: str, optional
        :return: Representation of the user as a Python dictionary.
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        include_sessions = kwargs.get("include_sessions", False)
        include_token = kwargs.get("include_token", False)
        custom_attributes = kwargs.get("customAttributes", None)
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        if id is not None:
            self.opencti.admin_logger.info("Fetching user with ID", {"id": id})
            query = (
                """
                query UserRead($id: String!) {
                    user(id: $id) {
                        """
                + (self.properties if custom_attributes is None else custom_attributes)
                + (self.token_properties if include_token else "")
                + (self.session_properties if include_sessions else "")
                + """
                    }
                }
                """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(result["data"]["user"])
        elif filters is not None or search is not None:
            results = self.list(
                filters=filters,
                search=search,
                include_sessions=include_sessions,
                customAttributes=custom_attributes,
            )
            user = results[0] if results else None
            if not include_token or user is None:
                return user
            else:
                return self.read(
                    id=user["id"],
                    include_sessions=include_sessions,
                    include_token=include_token,
                    customAttributes=custom_attributes,
                )
        else:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing paramters: id, search, or filters"
            )
            return None

    def create(self, **kwargs) -> Optional[Dict]:
        """Creates a new user with basic details

        Note that when SSO is connected users generally do not need to be
        manually created.

        Additionally note that if there is no password passed to this function
        then a random password will be created and will not be returned. This
        is useful for creating service accounts and connector accounts.

        :param name: Name to assign to the user.
        :type name: str
        :param user_email: Email address for the user.
        :type user_email: str
        :param password:  Password that should be assigned
            to the user. If one is not provided then a random one will be
            generated, defaults to None.
        :type password: str, optional
        :param firstname: First name of the user
        :type firstname: str, optional
        :param lastname: Last name of the user
        :type lastname: str, optional
        :param description: Description for the user
        :type description: str, optional
        :param language: Language the user should use
        :type language: str, optional
        :param theme: Theme to set for the user, either light or dark
        :type theme: str, optional
        :param objectOrganization: List of organization IDs to add the user to
        :type objectOgranization: List[str], optional
        :param account_status: The status of the account: Active, Expired,
            Inactive, or Locked
        :type account_status: str, optional
        :param account_lock_after_date: ISO 8901 of when account should be
            locked
        :type account_lock_after_date: str, optional
        :param unit_system: Unit system for the user, metric or imperial
        :type unit_system: str, optional
        :param submenu_show_icons: Defaults to False. Whether or not to show
            icons in submenus on the left hand menu bar in the UI
        :type submenu_show_icons: bool, optional
        :param submenu_auto_collaps: Defaults to False. Whether to auto-
            collapse the left hand menu bar in the UI
        :type submenu_auto_collapse: bool, optional
        :param monochrome_labels: Defaults to False. Whether to ignore colours
            and just show entity labels in monochrome.
        :type monochrome_labels: bool, optional
        :param groups: List of group IDs to add the user to
        :type groups: List[str], optional
        :param user_confidence_level: Confidence level object to assign to the
            user. This may not impact effective confidence depending on group
            membership.
        :type user_confidence_level: Dict
        :param customAttributes: Custom attributes to return for the user
        :type customAttributes: str, optional
        :param include_token: Defaults to False. Whether to include the API
            token for the new user in the response.
        :type include_token: bool, optional
        :return: Representation of the user without sessions or API token.
        :rtype: Optional[Dict]
        """
        name = kwargs.get("name", None)
        user_email = kwargs.get("user_email", None)
        password = kwargs.get("password", None)
        firstname = kwargs.get("firstname", None)
        lastname = kwargs.get("lastname", None)
        description = kwargs.get("description", None)
        language = kwargs.get("language", None)
        theme = kwargs.get("theme", None)
        object_organization = kwargs.get("objectOrganization", None)
        account_status = kwargs.get("account_status", None)
        account_lock_after_date = kwargs.get("account_lock_after_date", None)
        unit_system = kwargs.get("unit_system", None)
        submenu_show_icons = kwargs.get("submenu_show_icons", False)
        submenu_auto_collapse = kwargs.get("submenu_auto_collapse", False)
        monochrome_labels = kwargs.get("monochrome_labels", False)
        groups = kwargs.get("groups", None)
        user_confidence_level = kwargs.get("user_confidence_level", None)
        custom_attributes = kwargs.get("customAttributes", None)
        include_token = kwargs.get("include_token", False)

        if name is None or user_email is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: name and user_email"
            )
            return None

        self.opencti.admin_logger.info(
            "Creating a new user", {"name": name, "email": user_email}
        )
        if password is None:
            self.opencti.admin_logger.info(
                "Generating random password for user",
                {"name": name, "user_email": user_email},
            )
            password = secrets.token_urlsafe(64)
        query = (
            """
            mutation UserAdd($input: UserAddInput!) {
                userAdd(input: $input) {
                    """
            + (self.properties if custom_attributes is None else custom_attributes)
            + (self.token_properties if include_token else "")
            + """
                }
            }
            """
        )
        result = self.opencti.query(
            query,
            {
                "input": {
                    "user_email": user_email,
                    "name": name,
                    "password": password,
                    "firstname": firstname,
                    "lastname": lastname,
                    "description": description,
                    "language": language,
                    "theme": theme,
                    "objectOrganization": object_organization,
                    "account_status": account_status,
                    "account_lock_after_date": account_lock_after_date,
                    "unit_system": unit_system,
                    "submenu_show_icons": submenu_show_icons,
                    "submenu_auto_collapse": submenu_auto_collapse,
                    "monochrome_labels": monochrome_labels,
                    "groups": groups,
                    "user_confidence_level": user_confidence_level,
                }
            },
        )
        return self.opencti.process_multiple_fields(result["data"]["userAdd"])

    def delete(self, **kwargs):
        """Deletes the given user from the platform.

        :param id: ID of the user to delete.
        :type id: str
        """
        id = kwargs.get("id", None)
        if id is None:
            self.opencti.admin_logger.error("[opencti_user] Missing parameter: id")
            return None

        self.opencti.admin_logger.info("Deleting user", {"id": id})
        query = """
            mutation DeleteUser($id: ID!) {
                userEdit(id: $id) {
                    delete
                }
            }
        """
        self.opencti.query(query, {"id": id})

    def me(self, **kwargs) -> Dict:
        """Reads the currently authenticated user.

        :param include_token:  Whether to inclued the API
            token of the currently authenticated user, defaults to False.
        :type include_token: bool, optional
        :param customAttributes: Custom attributes to return on the User
        :type customAttributes: str, optional
        :return: Representation of the user.
        :rtype: dict
        """
        include_token = kwargs.get("include_token", False)
        custom_attributes = kwargs.get("customAttributes", None)

        self.opencti.admin_logger.info("Reading MeUser")
        query = (
            """
            query Me {
                me {
                    """
            + (self.me_properties if custom_attributes is None else custom_attributes)
            + (self.token_properties if include_token else "")
            + """
                }
            }
            """
        )
        result = self.opencti.query(query)
        return self.opencti.process_multiple_fields(result["data"]["me"])

    def update_field(self, **kwargs) -> Optional[Dict]:
        """Update a given user using fieldPatch

        :param id: ID of the user to update.
        :type id: str
        :param input: FieldPatchInput objects to edit user
        :type input: List[Dict]
        :param customAttributes: Custom attributes to return from the mutation
        :type customAttributes: str, optional
        :return: Representation of the user without sessions or API token.
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        input = kwargs.get("input", None)
        custom_attributes = kwargs.get("customAttributes", None)
        if id is None or input is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: id and input"
            )
            return None

        self.opencti.admin_logger.info(
            "Editing user with input (not shown to hide password and API token"
            " changes)",
            {"id": id},
        )
        query = (
            """
            mutation UserEdit($id: ID!, $input: [EditInput]!) {
                userEdit(id: $id) {
                    fieldPatch(input: $input) {
                        """
            + (self.properties if custom_attributes is None else custom_attributes)
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"id": id, "input": input})
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["fieldPatch"]
        )

    def add_membership(self, **kwargs) -> Optional[Dict]:
        """Adds the user to a given group.

        :param id: User ID to add to the group.
        :type id: str
        :param group_id: Group ID to add the user to.
        :type group_id: str
        :return: Representation of the InternalRelationship
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        group_id = kwargs.get("group_id", None)
        if id is None or group_id is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: id and group_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Adding user to group", {"id": id, "group_id": group_id}
        )
        query = """
            mutation UserEditAddMembership($id: ID!, $group_id: ID!) {
                userEdit(id: $id) {
                    relationAdd(input: {
                        relationship_type: "member-of",
                        toId: $group_id
                    }) {
                        id
                        from {
                            ... on User {
                                id, name, user_email
                            }
                        }
                        to {
                            ... on Group {
                                id, name, description
                            }
                        }
                    }
                }
            }
        """
        result = self.opencti.query(query, {"id": id, "group_id": group_id})
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["relationAdd"]
        )

    def delete_membership(self, **kwargs) -> Optional[Dict]:
        """Removes the user from the given group.

        :param id: User ID to remove from the group.
        :type id: str
        :param group_id: Group ID to remove the user from.
        :type group_id: str
        :return: Representation of the user without sessions or API token
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        group_id = kwargs.get("group_id", None)
        if id is None or group_id is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: id and group_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Removing used from group", {"id": id, "group_id": group_id}
        )
        query = (
            """
            mutation UserEditDeleteMembership($id: ID!, $group_id: StixRef!) {
                userEdit(id: $id) {
                    relationDelete(toId: $group_id, relationship_type: "member-of") {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"id": id, "group_id": group_id})
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["relationDelete"]
        )

    def add_organization(self, **kwargs) -> Optional[Dict]:
        """Adds a user to an organization

        :param id: User ID to add to organization
        :type id: str
        :param organization_id: ID of organization to add to
        :type organization_id: str
        :return: Representation of user without sessions or API key
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        organization_id = kwargs.get("organization_id", None)
        if id is None or organization_id is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: id and organization_id"
            )

        self.opencti.admin_logger.info(
            "Adding user to organization",
            {"id": id, "organization_id": organization_id},
        )
        query = (
            """
            mutation UserEditAddOrganization($id: ID!, $organization_id: ID!) {
                userEdit(id: $id) {
                    organizationAdd(organizationId: $organization_id) {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(
            query, {"id": id, "organization_id": organization_id}
        )
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["organizationAdd"]
        )

    def delete_organization(self, **kwargs) -> Optional[Dict]:
        """Delete a user from an organization

        :param id: User ID to remove from organization
        :type id: str
        :param organization_id: ID of organization to remove from
        :type organization_id: str
        :return: Representation of user without sessions or API key
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        organization_id = kwargs.get("organization_id", None)
        if id is None or organization_id is None:
            self.opencti.admin_logger.error(
                "[opencti_user] Missing parameters: id and organization_id"
            )

        self.opencti.admin_logger.info(
            "Removing user from organization",
            {"id": id, "organization_id": organization_id},
        )
        query = (
            """
            mutation UserEditDeleteOrganization($id: ID!, $organization_id: ID!) {
                userEdit(id: $id) {
                    organizationDelete(organizationId: $organization_id) {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(
            query, {"id": id, "organization_id": organization_id}
        )
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["organizationDelete"]
        )

    def token_renew(self, **kwargs) -> Optional[Dict]:
        """Rotates the API token for the given user

        :param user: User ID to rotate API token for.
        :type user: str
        :param include_token:  Whether to include new API
            token in response from server, defaults to False.
        :type include_token: bool, optional
        :return: Representation of user
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        include_token = kwargs.get("include_token", False)
        if id is None:
            self.opencti.admin_logger.error("[opencti_user] Missing parameter: id")
            return None

        self.opencti.admin_logger.info("Rotating API key for user", {"id": id})
        query = (
            """
            mutation UserEditRotateToken($id: ID!) {
                userEdit(id: $id) {
                    tokenRenew {
                        """
            + self.properties
            + (self.token_properties if include_token else "")
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"id": id})
        return self.opencti.process_multiple_fields(
            result["data"]["userEdit"]["tokenRenew"]
        )

    def process_multiple_fields(self, data):
        if "roles" in data:
            data["roles"] = self.opencti.process_multiple(data["roles"])
            data["rolesIds"] = self.opencti.process_multiple_ids(data["roles"])
        if "groups" in data:
            data["groups"] = self.opencti.process_multiple(data["groups"])
            data["groupsIds"] = self.opencti.process_multiple_ids(data["groups"])
        if "objectOrganization" in data:
            data["objectOrganization"] = self.opencti.process_multiple(
                data["objectOrganization"]
            )
            data["objectOrganizationIds"] = self.opencti.process_multiple_ids(
                data["objectOrganization"]
            )

        return data



class OpenCTIApiInternalFile:
    """OpenCTIApiInternalFile"""

    def __init__(self, api):
        self.api = api

    def delete(self, **kwargs):
        item = kwargs.get("item", None)
        file_name = self.api.get_attribute_in_extension("id", item)
        if file_name is not None:
            query = """
                mutation InternalFileDelete($fileName: String) {
                    deleteImport(fileName: $fileName)
                }
               """
            self.api.query(
                query,
                {
                    "fileName": file_name,
                },
            )
        else:
            self.api.app_logger.error(
                "[stix_internal_file] Cant delete internal file, missing parameters: fileName"
            )
            return None



# coding: utf-8

import datetime
import json
import uuid

from stix2.canonicalization.Canonicalize import canonicalize


class Note:
    def __init__(self, opencti):
        self.opencti = opencti
        self.properties = """
            id
            standard_id
            entity_type
            parent_types
            spec_version
            created_at
            updated_at
            status {
                id
                template {
                  id
                  name
                  color
                }
            }
            createdBy {
                ... on Identity {
                    id
                    standard_id
                    entity_type
                    parent_types
                    spec_version
                    identity_class
                    name
                    description
                    roles
                    contact_information
                    x_opencti_aliases
                    created
                    modified
                    objectLabel {
                        id
                        value
                        color
                    }
                }
                ... on Organization {
                    x_opencti_organization_type
                    x_opencti_reliability
                }
                ... on Individual {
                    x_opencti_firstname
                    x_opencti_lastname
                }
            }
            objectOrganization {
                id
                standard_id
                name
            }
            objectMarking {
                id
                standard_id
                entity_type
                definition_type
                definition
                created
                modified
                x_opencti_order
                x_opencti_color
            }
            objectLabel {
                id
                value
                color
            }
            externalReferences {
                edges {
                    node {
                        id
                        standard_id
                        entity_type
                        source_name
                        description
                        url
                        hash
                        external_id
                        created
                        modified
                    }
                }
            }
            revoked
            confidence
            created
            modified
            attribute_abstract
            content
            authors
            note_types
            likelihood
            objects(all: true) {
                edges {
                    node {
                        ... on BasicObject {
                            id
                            entity_type
                            parent_types
                        }
                        ... on BasicRelationship {
                            id
                            entity_type
                            parent_types
                        }
                        ... on StixObject {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                        }
                        ... on AttackPattern {
                            name
                        }
                        ... on Campaign {
                            name
                        }
                        ... on CourseOfAction {
                            name
                        }
                        ... on Individual {
                            name
                        }
                        ... on Organization {
                            name
                        }
                        ... on Sector {
                            name
                        }
                        ... on System {
                            name
                        }
                        ... on Indicator {
                            name
                        }
                        ... on Infrastructure {
                            name
                        }
                        ... on IntrusionSet {
                            name
                        }
                        ... on Position {
                            name
                        }
                        ... on City {
                            name
                        }
                        ... on Country {
                            name
                        }
                        ... on Region {
                            name
                        }
                        ... on Malware {
                            name
                        }
                        ... on ThreatActor {
                            name
                        }
                        ... on Tool {
                            name
                        }
                        ... on Vulnerability {
                            name
                        }
                        ... on Incident {
                            name
                        }
                        ... on Event {
                            name
                        }
                        ... on Channel {
                            name
                        }
                        ... on Narrative {
                            name
                        }
                        ... on Language {
                            name
                        }
                        ... on DataComponent {
                            name
                        }
                        ... on DataSource {
                            name
                        }
                        ... on Case {
                            name
                        }
                        ... on StixCyberObservable {
                            observable_value
                        }                        
                        ... on StixCoreRelationship {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                            relationship_type
                        }
                       ... on StixSightingRelationship {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                        }                        
                    }
                }
            }
        """
        self.properties_with_files = """
            id
            standard_id
            entity_type
            parent_types
            spec_version
            created_at
            updated_at
            status {
                id
                template {
                  id
                  name
                  color
                }
            }
            createdBy {
                ... on Identity {
                    id
                    standard_id
                    entity_type
                    parent_types
                    spec_version
                    identity_class
                    name
                    description
                    roles
                    contact_information
                    x_opencti_aliases
                    created
                    modified
                    objectLabel {
                        id
                        value
                        color
                    }
                }
                ... on Organization {
                    x_opencti_organization_type
                    x_opencti_reliability
                }
                ... on Individual {
                    x_opencti_firstname
                    x_opencti_lastname
                }
            }
            objectMarking {
                id
                standard_id
                entity_type
                definition_type
                definition
                created
                modified
                x_opencti_order
                x_opencti_color
            }
            objectLabel {
                id
                value
                color
            }
            externalReferences {
                edges {
                    node {
                        id
                        standard_id
                        entity_type
                        source_name
                        description
                        url
                        hash
                        external_id
                        created
                        modified
                        importFiles {
                            edges {
                                node {
                                    id
                                    name
                                    size
                                    metaData {
                                        mimetype
                                        version
                                    }
                                }
                            }
                        }
                    }
                }
            }
            revoked
            confidence
            created
            modified
            attribute_abstract
            content
            authors
            note_types
            likelihood
            objects(all: true) {
                edges {
                    node {
                        ... on BasicObject {
                            id
                            entity_type
                            parent_types
                        }
                        ... on BasicRelationship {
                            id
                            entity_type
                            parent_types
                        }
                        ... on StixObject {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                        }
                        ... on AttackPattern {
                            name
                        }
                        ... on Campaign {
                            name
                        }
                        ... on CourseOfAction {
                            name
                        }
                        ... on Individual {
                            name
                        }
                        ... on Organization {
                            name
                        }
                        ... on Sector {
                            name
                        }
                        ... on System {
                            name
                        }
                        ... on Indicator {
                            name
                        }
                        ... on Infrastructure {
                            name
                        }
                        ... on IntrusionSet {
                            name
                        }
                        ... on Position {
                            name
                        }
                        ... on City {
                            name
                        }
                        ... on Country {
                            name
                        }
                        ... on Region {
                            name
                        }
                        ... on Malware {
                            name
                        }
                        ... on ThreatActor {
                            name
                        }
                        ... on Tool {
                            name
                        }
                        ... on Vulnerability {
                            name
                        }
                        ... on Incident {
                            name
                        }
                        ... on Event {
                            name
                        }
                        ... on Channel {
                            name
                        }
                        ... on Narrative {
                            name
                        }
                        ... on Language {
                            name
                        }
                        ... on DataComponent {
                            name
                        }
                        ... on DataSource {
                            name
                        }
                        ... on Case {
                            name
                        }
                        ... on StixCyberObservable {
                            observable_value
                        }                        
                        ... on StixCoreRelationship {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                            relationship_type
                        }
                       ... on StixSightingRelationship {
                            standard_id
                            spec_version
                            created_at
                            updated_at
                        }                        
                    }
                }
            }
            importFiles {
                edges {
                    node {
                        id
                        name
                        size
                        metaData {
                            mimetype
                            version
                        }
                    }
                }
            }
        """

    @staticmethod
    def generate_id(created, content):
        if content is None:
            raise ValueError("content is required")
        if created is not None:
            if isinstance(created, datetime.datetime):
                created = created.isoformat()
            data = {"content": content.strip(), "created": created}
        else:
            data = {"content": content.strip()}
        data = canonicalize(data, utf8=False)
        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
        return "note--" + id

    @staticmethod
    def generate_id_from_data(data):
        return Note.generate_id(data.get("created"), data["content"])

    """
        List Note objects

        :param filters: the filters to apply
        :param search: the search keyword
        :param first: return the first n rows from the after ID (or the beginning if not set)
        :param after: ID of the first row for pagination
        :return List of Note objects
    """

    def list(self, **kwargs):
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        first = kwargs.get("first", 100)
        after = kwargs.get("after", None)
        order_by = kwargs.get("orderBy", None)
        order_mode = kwargs.get("orderMode", None)
        custom_attributes = kwargs.get("customAttributes", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("withPagination", False)
        with_files = kwargs.get("withFiles", False)

        self.opencti.app_logger.info(
            "Listing Notes with filters", {"filters": json.dumps(filters)}
        )
        query = (
            """
            query Notes($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: NotesOrdering, $orderMode: OrderingMode) {
                notes(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                    edges {
                        node {
                            """
            + (
                custom_attributes
                if custom_attributes is not None
                else (self.properties_with_files if with_files else self.properties)
            )
            + """
                        }
                    }
                    pageInfo {
                        startCursor
                        endCursor
                        hasNextPage
                        hasPreviousPage
                        globalCount
                    }
                }
            }
        """
        )
        result = self.opencti.query(
            query,
            {
                "filters": filters,
                "search": search,
                "first": first,
                "after": after,
                "orderBy": order_by,
                "orderMode": order_mode,
            },
        )
        if get_all:
            final_data = []
            data = self.opencti.process_multiple(result["data"]["notes"])
            final_data = final_data + data
            while result["data"]["notes"]["pageInfo"]["hasNextPage"]:
                after = result["data"]["notes"]["pageInfo"]["endCursor"]
                self.opencti.app_logger.info("Listing Notes", {"after": after})
                result = self.opencti.query(
                    query,
                    {
                        "filters": filters,
                        "search": search,
                        "first": first,
                        "after": after,
                        "orderBy": order_by,
                        "orderMode": order_mode,
                    },
                )
                data = self.opencti.process_multiple(result["data"]["notes"])
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["notes"], with_pagination
            )

    """
        Read a Note object

        :param id: the id of the Note
        :param filters: the filters to apply if no id provided
        :return Note object
    """

    def read(self, **kwargs):
        id = kwargs.get("id", None)
        filters = kwargs.get("filters", None)
        custom_attributes = kwargs.get("customAttributes", None)
        with_files = kwargs.get("withFiles", False)
        if id is not None:
            self.opencti.app_logger.info("Reading Note", {"id": id})
            query = (
                """
                query Note($id: String!) {
                    note(id: $id) {
                        """
                + (
                    custom_attributes
                    if custom_attributes is not None
                    else (self.properties_with_files if with_files else self.properties)
                )
                + """
                    }
                }
            """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(result["data"]["note"])
        elif filters is not None:
            result = self.list(filters=filters)
            if len(result) > 0:
                return result[0]
            else:
                return None

    """
        Check if a note already contains a STIX entity

        :return Boolean
    """

    def contains_stix_object_or_stix_relationship(self, **kwargs):
        id = kwargs.get("id", None)
        stix_object_or_stix_relationship_id = kwargs.get(
            "stixObjectOrStixRelationshipId", None
        )
        if id is not None and stix_object_or_stix_relationship_id is not None:
            self.opencti.app_logger.info(
                "Checking StixObjectOrStixRelationship in Note",
                {
                    "id": id,
                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
                },
            )
            query = """
                query NoteContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
                    noteContainsStixObjectOrStixRelationship(id: $id, stixObjectOrStixRelationshipId: $stixObjectOrStixRelationshipId)
                }
            """
            result = self.opencti.query(
                query,
                {
                    "id": id,
                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
                },
            )
            return result["data"]["noteContainsStixObjectOrStixRelationship"]
        else:
            self.opencti.app_logger.error(
                "[opencti_note] Missing parameters: id or entity_id"
            )

    """
        Create a Note object

        :param name: the name of the Note
        :return Note object
    """

    def create(self, **kwargs):
        stix_id = kwargs.get("stix_id", None)
        created_by = kwargs.get("createdBy", None)
        objects = kwargs.get("objects", None)
        object_marking = kwargs.get("objectMarking", None)
        object_label = kwargs.get("objectLabel", None)
        external_references = kwargs.get("externalReferences", None)
        revoked = kwargs.get("revoked", None)
        confidence = kwargs.get("confidence", None)
        lang = kwargs.get("lang", None)
        created = kwargs.get("created", None)
        modified = kwargs.get("modified", None)
        abstract = kwargs.get("abstract", None)
        content = kwargs.get("content", None)
        authors = kwargs.get("authors", None)
        note_types = kwargs.get("note_types", None)
        likelihood = kwargs.get("likelihood", None)
        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
        granted_refs = kwargs.get("objectOrganization", None)
        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
        update = kwargs.get("update", False)

        if content is not None:
            self.opencti.app_logger.info("Creating Note", {"content": content})
            query = """
                mutation NoteAdd($input: NoteAddInput!) {
                    noteAdd(input: $input) {
                        id
                        standard_id
                        entity_type
                        parent_types
                    }
                }
            """
            result = self.opencti.query(
                query,
                {
                    "input": {
                        "stix_id": stix_id,
                        "createdBy": created_by,
                        "objectMarking": object_marking,
                        "objectLabel": object_label,
                        "objectOrganization": granted_refs,
                        "objects": objects,
                        "externalReferences": external_references,
                        "revoked": revoked,
                        "confidence": confidence,
                        "lang": lang,
                        "created": created,
                        "modified": modified,
                        "attribute_abstract": abstract,
                        "content": content,
                        "authors": authors,
                        "note_types": note_types,
                        "likelihood": likelihood,
                        "x_opencti_stix_ids": x_opencti_stix_ids,
                        "x_opencti_workflow_id": x_opencti_workflow_id,
                        "update": update,
                    }
                },
            )
            return self.opencti.process_multiple_fields(result["data"]["noteAdd"])
        else:
            self.opencti.app_logger.error("[opencti_note] Missing parameters: content")

    """
        Add a Stix-Entity object to Note object (object_refs)

        :param id: the id of the Note
        :param entity_id: the id of the Stix-Entity
        :return Boolean
    """

    def add_stix_object_or_stix_relationship(self, **kwargs):
        id = kwargs.get("id", None)
        stix_object_or_stix_relationship_id = kwargs.get(
            "stixObjectOrStixRelationshipId", None
        )
        if id is not None and stix_object_or_stix_relationship_id is not None:
            if self.contains_stix_object_or_stix_relationship(
                id=id,
                stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
            ):
                return True
            self.opencti.app_logger.info(
                "Adding StixObjectOrStixRelationship to Note",
                {
                    "id": id,
                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
                },
            )
            query = """
               mutation NoteEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                   noteEdit(id: $id) {
                        relationAdd(input: $input) {
                            id
                        }
                   }
               }
            """
            self.opencti.query(
                query,
                {
                    "id": id,
                    "input": {
                        "toId": stix_object_or_stix_relationship_id,
                        "relationship_type": "object",
                    },
                },
            )
            return True
        else:
            self.opencti.app_logger.error(
                "[opencti_note] Missing parameters: id and stix_object_or_stix_relationship_id",
            )
            return False

    """
        Remove a Stix-Entity object to Note object (object_refs)

        :param id: the id of the Note
        :param entity_id: the id of the Stix-Entity
        :return Boolean
    """

    def remove_stix_object_or_stix_relationship(self, **kwargs):
        id = kwargs.get("id", None)
        stix_object_or_stix_relationship_id = kwargs.get(
            "stixObjectOrStixRelationshipId", None
        )
        if id is not None and stix_object_or_stix_relationship_id is not None:
            self.opencti.app_logger.info(
                "Removing StixObjectOrStixRelationship in Note",
                {
                    "id": id,
                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
                },
            )
            query = """
               mutation NotetEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                   noteEdit(id: $id) {
                        relationDelete(toId: $toId, relationship_type: $relationship_type) {
                            id
                        }
                   }
               }
            """
            self.opencti.query(
                query,
                {
                    "id": id,
                    "toId": stix_object_or_stix_relationship_id,
                    "relationship_type": "object",
                },
            )
            return True
        else:
            self.opencti.app_logger.error(
                "[opencti_note] Missing parameters: id and entity_id"
            )
            return False

    """
        Import a Note object from a STIX2 object

        :param stixObject: the Stix-Object Note
        :return Note object
    """

    def import_from_stix2(self, **kwargs):
        stix_object = kwargs.get("stixObject", None)
        extras = kwargs.get("extras", {})
        update = kwargs.get("update", False)
        if stix_object is not None:
            # Search in extensions
            if "x_opencti_stix_ids" not in stix_object:
                stix_object["x_opencti_stix_ids"] = (
                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
                )
            if "x_opencti_granted_refs" not in stix_object:
                stix_object["x_opencti_granted_refs"] = (
                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
                )
            if "x_opencti_workflow_id" not in stix_object:
                stix_object["x_opencti_workflow_id"] = (
                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                )

            return self.create(
                stix_id=stix_object["id"],
                createdBy=(
                    extras["created_by_id"] if "created_by_id" in extras else None
                ),
                objectMarking=(
                    extras["object_marking_ids"]
                    if "object_marking_ids" in extras
                    else None
                ),
                objectLabel=(
                    extras["object_label_ids"] if "object_label_ids" in extras else None
                ),
                objects=extras["object_ids"] if "object_ids" in extras else [],
                externalReferences=(
                    extras["external_references_ids"]
                    if "external_references_ids" in extras
                    else None
                ),
                revoked=stix_object["revoked"] if "revoked" in stix_object else None,
                confidence=(
                    stix_object["confidence"] if "confidence" in stix_object else None
                ),
                lang=stix_object["lang"] if "lang" in stix_object else None,
                created=stix_object["created"] if "created" in stix_object else None,
                modified=stix_object["modified"] if "modified" in stix_object else None,
                abstract=(
                    self.opencti.stix2.convert_markdown(stix_object["abstract"])
                    if "abstract" in stix_object
                    else None
                ),
                content=(
                    self.opencti.stix2.convert_markdown(stix_object["content"])
                    if "content" in stix_object
                    else None
                ),
                x_opencti_stix_ids=(
                    stix_object["x_opencti_stix_ids"]
                    if "x_opencti_stix_ids" in stix_object
                    else None
                ),
                authors=stix_object["authors"] if "authors" in stix_object else None,
                note_types=(
                    stix_object["note_types"] if "note_types" in stix_object else None
                ),
                likelihood=(
                    stix_object["likelihood"] if "likelihood" in stix_object else None
                ),
                objectOrganization=(
                    stix_object["x_opencti_granted_refs"]
                    if "x_opencti_granted_refs" in stix_object
                    else None
                ),
                x_opencti_workflow_id=(
                    stix_object["x_opencti_workflow_id"]
                    if "x_opencti_workflow_id" in stix_object
                    else None
                ),
                update=update,
            )
        else:
            self.opencti.app_logger.error(
                "[opencti_note] Missing parameters: stixObject"
            )



from typing import Dict, List, Optional


class Group:
    """Representation of a Group in OpenCTI

    Groups have members and also have assigned roles. Roles attached to a group
    determine what members of the group have permissions to do according to
    the capabilities the role has.

    Additionally, groups have a confidence level which informs the effective
    confidence of members of the group.

    Groups also have permissions on Marking Definitions. Assigned marking
    definitions allow users to apply their capabilities on objects with those
    definitions. Additionally, there are default markings added to all objects
    created by members of a group, and max shareable definitions which
    determine which objects users can export from the platform to share.

    See the properties attribute to understand what properties are fetched by
    default from GraphQL queries.
    """

    def __init__(self, opencti):
        self.opencti = opencti
        self.properties = """
            id
            standard_id
            name
            description
            entity_type
            parent_types
            created_at
            updated_at
            default_assignation
            no_creators
            restrict_delete
            default_hidden_types
            auto_new_marking
            allowed_marking {
                id, standard_id, definition_type, definition
            }
            default_marking {
                entity_type
                values {
                    id, standard_id, definition_type, definition
                }
            }
            not_shareable_marking_types
            max_shareable_marking {
                id, standard_id, definition_type, definition
            }
            group_confidence_level {
                max_confidence
                overrides {
                    entity_type
                    max_confidence
                }
            }
            roles {
                edges {
                    node {
                        id, name
                        capabilities {
                            id, name
                        }
                    }
                }
            }
            members {
                edges {
                    node {
                        id, individual_id, user_email, name
                    }
                }
            }
        """

    def list(self, **kwargs) -> List[Dict]:
        """Lists groups based on a number of filters.

        :param first:  Retrieve this number of results. If 0
            then fetches all results, defaults to 0.
        :type first: int, optional
        :param after:  ID of the group to fetch results
            after in the list of all results, defaults to None.
        :type after: str, optional
        :param orderBy:  Field by which to order results.
            Must be one of name, default_assignation, no_creators,
            restrict_delete, auto_new_marking, created_at, updated_at,
            group_confidence_level, and _score, defaults to "name".
        :type orderBy: str, optional
        :param orderMode:  Direction of ordering. Must be
            one of "asc" or "desc", defaults to "asc".
        :type orderMode: str, optional
        :param search:  String to search groups for, defaults to None.
        :type search: str, optional
        :param filters:  OpenCTI API FilterGroup object.
            This is an advanced parameter. To learn more please search for
            the FilterGroup object in the OpenCTI GraphQL Playground, defaults
            to {}.
        :type filters: dict, optional
        :param customAttributes: Custom attributes to fetch from the GraphQL
            query
        :type customAttributes: str, optional
        :param getAll: Defaults to False. Whether or not to get all results
            from the search. If True then param first is ignored.
        :type getAll: bool, optional
        :param withPagination: Defaults to False. Whether to return pagination
            info with results.
        :type withPagination: bool, optional
        :return: List of groups in dictionary representation.
        :rtype: list[dict]
        """
        first = kwargs.get("first", 500)
        after = kwargs.get("after", None)
        order_by = kwargs.get("orderBy", None)
        order_mode = kwargs.get("orderMode", None)
        search = kwargs.get("search", None)
        filters = kwargs.get("filters", None)
        custom_attributes = kwargs.get("customAttributes", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("withPagination", False)

        if get_all:
            first = 100

        self.opencti.admin_logger.info(
            "Fetching groups with filters", {"filters": filters}
        )
        query = (
            """
            query Groups($first: Int, $after: ID, $orderBy: GroupsOrdering, $orderMode: OrderingMode, $search: String, $filters: FilterGroup) {
                groups(first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode, search: $search, filters: $filters) {
                    edges {
                        node {
                                """
            + (self.properties if custom_attributes is None else custom_attributes)
            + """
                        }
                    }
                    pageInfo {
                        startCursor
                        endCursor
                        hasNextPage
                        hasPreviousPage
                        globalCount
                    }
                }
            }
            """
        )
        result = self.opencti.query(
            query,
            {
                "first": first,
                "after": after,
                "orderBy": order_by,
                "orderMode": order_mode,
                "search": search,
                "filters": filters,
            },
        )

        if get_all:
            final_data = []
            data = self.opencti.process_multiple(result["data"]["groups"])
            final_data = final_data + data
            while result["data"]["groups"]["pageInfo"]["hasNextPage"]:
                after = result["data"]["groups"]["pageInfo"]["endCursor"]
                result = self.opencti.query(
                    query,
                    {
                        "first": first,
                        "after": after,
                        "orderBy": order_by,
                        "orderMode": order_mode,
                        "search": search,
                        "filters": filters,
                    },
                )
                data = self.opencti.process_multiple(result["data"]["groups"])
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["groups"], with_pagination
            )

    def read(self, **kwargs) -> Optional[Dict]:
        """Fetch a given group from OpenCTI

        One of id or filters is required.

        :param id: ID of the group to fetch
        :type id: str, optional
        :param filters: Filters to apply to find single group
        :type filters: dict, optional
        :param customAttributes: Custom attributes to fetch for the group
        :type customAttributes: str
        :return: Representation of a group.
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        custom_attributes = kwargs.get("customAttributes", None)
        if id is not None:
            self.opencti.admin_logger.info("Fetching group with ID", {"id": id})
            query = (
                """
                query Group($id: String!) {
                    group(id: $id) {
                        """
                + (self.properties if custom_attributes is None else custom_attributes)
                + """
                    }
                }
                """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(result["data"]["group"])
        elif filters is not None or search is not None:
            results = self.list(
                filters=filters, search=search, customAttributes=custom_attributes
            )
            return results[0] if results else None
        else:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id or filters"
            )
            return None

    def create(self, **kwargs) -> Optional[Dict]:
        """Create a group with required details

        Groups can be configured after creation using other functions.

        :param name: Name of the group to create.
        :type name: str
        :param id_confidence_level: Confidence-level dictionary, with a
            max_confidence member between 0 and 100 (incl) and an overrides
            list with max_confidence and the entity_type it applies to.
        :type id_confidence_level: dict
        :param description: Description of the group
        :type description: str, optional
        :param default_assignation: Defaults to False. Whether or not to assign
            this group by default to all new users.
        :type default_assignation: bool, optional
        :param no_creators: Defaults to False. Whether or not to create
            authors for members of this group.
        :type no_creators: bool, optional
        :param restrict_delete: Defaults to False. Whether or not to restrict
            members deleting entities that are not their own.
        :type restrict_delete: bool, optional
        :param auto_new_marking: Defaults to False. Whether or not to allow
            members access to new markings automatically.
        :type auto_new_marking: bool, optional
        :param customAttributes: Attributes to retrieve from the new group
        :type customAttributes: str, optional
        :return: Representation of the group.
        :rtype: Optional[Dict]
        """
        name = kwargs.get("name", None)
        group_confidence_level = kwargs.get("group_confidence_level", None)
        description = kwargs.get("description", None)
        default_assignation = kwargs.get("default_assignation", False)
        no_creators = kwargs.get("no_creators", False)
        restrict_delete = kwargs.get("restrict_delete", False)
        auto_new_marking = kwargs.get("auto_new_marking", False)
        custom_attributes = kwargs.get("customAttributes", None)

        if name is None or group_confidence_level is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: name and group_confidence_level"
            )
            return None

        self.opencti.admin_logger.info(
            "Creating new group with parameters",
            {
                "name": name,
                "group_confidence_level": group_confidence_level,
                "description": description,
                "default_assignation": default_assignation,
                "no_creators": no_creators,
                "restrict_delete": restrict_delete,
                "auto_new_marking": auto_new_marking,
            },
        )
        query = (
            """
            mutation GroupAdd($input: GroupAddInput!) {
                groupAdd(input: $input) {
                    """
            + (self.properties if custom_attributes is None else custom_attributes)
            + """
                }
            }
            """
        )
        result = self.opencti.query(
            query,
            {
                "input": {
                    "name": name,
                    "description": description,
                    "default_assignation": default_assignation,
                    "no_creators": no_creators,
                    "restrict_delete": restrict_delete,
                    "auto_new_marking": auto_new_marking,
                    "group_confidence_level": group_confidence_level,
                }
            },
        )
        return self.opencti.process_multiple_fields(result["data"]["groupAdd"])

    def delete(self, **kwargs):
        """Delete a given group from OpenCTI

        :param id: ID of the group to delete.
        :type id: str
        """
        id = kwargs.get("id", None)
        if id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Cant delete group, missing parameter: id"
            )
            return None
        self.opencti.admin_logger.info("Deleting group", {"id": id})
        query = """
            mutation GroupDelete($id: ID!) {
                groupEdit(id: $id) {
                    delete
                }
            }
        """
        self.opencti.query(query, {"id": id})

    def update_field(self, **kwargs) -> Optional[Dict]:
        """Update a group using fieldPatch

        :param id: ID of the group to update
        :type id: str
        :param input: FieldPatchInput object to edit group
        :type input: List[Dict]
        :param customAttributes: Custom attributes to retrieve from group
        :type customAttribues: str, optional
        :return: Representation of a group
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        input = kwargs.get("input", None)
        custom_attributes = kwargs.get("customAttributes", None)

        if id is None or input is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and input"
            )
            return None

        self.opencti.admin_logger.info("Editing group with input", {"input": input})
        query = (
            """
            mutation GroupEdit($id: ID!, $input:[EditInput]!) {
                groupEdit(id: $id) {
                    fieldPatch(input: $input) {
                        """
            + (self.properties if custom_attributes is None else custom_attributes)
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"id": id, "input": input})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["fieldPatch"]
        )

    def add_member(self, **kwargs) -> Optional[Dict]:
        """Add a member to a given group.

        :param id: ID of the group to add a member to
        :type id: str
        :param user_id: ID to add to the group
        :type user_id: str
        :return: Representation of the relationship
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        user_id = kwargs.get("user_id", None)

        if id is None or user_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and user_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Adding member to group", {"groupId": id, "userId": user_id}
        )
        query = """
            mutation GroupEditMemberAdd($groupId: ID!, $userId: ID!) {
                groupEdit(id: $groupId) {
                    relationAdd(input: {
                        fromId: $userId,
                        relationship_type: "member-of"
                    }) {
                        id, standard_id, entity_type, created_at, updated_at
                        from {
                            id, entity_type
                        }
                        to {
                            id, entity_type
                        }
                    }
                }
            }
            """
        result = self.opencti.query(query, {"groupId": id, "userId": user_id})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationAdd"]
        )

    def delete_member(self, **kwargs) -> Optional[Dict]:
        """Remove a given user from a group

        :param id: ID to remove a user from
        :type id: str
        :param user: ID to remove from the group
        :type user: str
        :return: Representation of the group after the member has been removed
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        user_id = kwargs.get("user_id", None)

        if id is None or user_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and user_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Removing member from group", {"groupId": id, "userId": user_id}
        )
        query = (
            """
            mutation GroupEditMemberDelete ($groupId: ID!, $userId: StixRef!) {
                groupEdit(id: $groupId) {
                    relationDelete(fromId: $userId, relationship_type: "member-of") {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"groupId": id, "userId": user_id})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationDelete"]
        )

    def add_role(self, **kwargs) -> Optional[Dict]:
        """Add a role to a given group

        :param id: ID to add a role to
        :type id: str
        :param role_id: Role ID to add to the group
        :type role: str
        :return: Representation of the group after a role has been added
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        role_id = kwargs.get("role_id", None)

        if id is None or role_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and role_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Adding role to group", {"groupId": id, "roleId": role_id}
        )
        query = """
            mutation GroupEditRoleAdd($groupId: ID!, $roleId: ID!) {
                groupEdit(id: $groupId) {
                    relationAdd(input: {
                        toId: $roleId, relationship_type: "has-role"
                    }) {
                        id, standard_id, entity_type, created_at, updated_at
                        from {
                            id, entity_type
                        }
                        to {
                            id, entity_type
                        }
                    }
                }
            }
            """
        result = self.opencti.query(query, {"groupId": id, "roleId": role_id})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationAdd"]
        )

    def delete_role(self, **kwargs) -> Optional[Dict]:
        """Removes a role from a given group

        :param id: ID to remove role from
        :type id: str
        :param role_id: Role ID to remove from the group
        :type role_id: str
        :return: Representation of the group after role is removed
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        role_id = kwargs.get("role_id", None)

        if id is None or role_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and role_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Removing role from group", {"groupId": id, "roleId": role_id}
        )
        query = (
            """
            mutation GroupEditRoleDelete($groupId: ID!, $roleId: StixRef!) {
                groupEdit(id: $groupId) {
                    relationDelete(toId: $roleId, relationship_type: "has-role") {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"groupId": id, "roleId": role_id})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationDelete"]
        )

    def edit_default_marking(self, **kwargs) -> Optional[Dict]:
        """Adds a default marking to the group.

        :param id: ID of the group.
        :type id: str
        :param marking_ids: IDs of the markings to add, or an empty list to
            remove all default markings
        :type marking_ids: List[str]
        :param entity:  STIX entity type to add default
            marking for. If set to "GLOBAL" applies to all entity types,
            defaults to "GLOBAL".
        :type entity: str, optional
        :return: Group after adding the default marking.
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        marking_ids = kwargs.get("marking_ids", None)
        entity_type = kwargs.get("entity_type", "GLOBAL")

        if id is None or marking_ids is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and marking_ids"
            )
            return None

        self.opencti.admin_logger.info(
            "Setting default markings for entity on group",
            {"markings": marking_ids, "entity_type": entity_type, "groupId": id},
        )
        query = (
            """
            mutation GroupEditEditDefaultMarking($id: ID!, $entity_type: String!, $values: [String!]) {
                groupEdit(id: $id) {
                    editDefaultMarking(input: {
                        entity_type: $entity_type,
                        values: $values
                    }) {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(
            query, {"id": id, "entity_type": entity_type, "values": marking_ids}
        )
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["editDefaultMarking"]
        )

    def add_allowed_marking(self, **kwargs) -> Optional[Dict]:
        """Allow a group to access a marking

        :param id: ID of group to authorise
        :type id: str
        :param marking_id: ID of marking to authorise
        :type marking_id: str
        :return: Relationship from the group to the marking definition
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        marking_id = kwargs.get("marking_id", None)

        if id is None or marking_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and marking_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Granting group access to marking definition",
            {"groupId": id, "markingId": marking_id},
        )
        query = """
            mutation GroupEditionMarkingsMarkingDefinitionsRelationAddMutation(
                $id: ID!
                $input: InternalRelationshipAddInput!
            ) {
                groupEdit(id: $id) {
                    relationAdd(input: $input) {
                        from {
                            __typename
                            ...GroupEditionMarkings_group
                            id
                        }
                        id
                        }
                    }
                }

                fragment GroupEditionMarkings_group on Group {
                id
                default_assignation
                allowed_marking {
                    id
                }
                not_shareable_marking_types
                max_shareable_marking {
                    id
                    definition
                    definition_type
                    x_opencti_order
                }
                default_marking {
                    entity_type
                    values {
                    id
                    }
                }
            }
        """
        result = self.opencti.query(
            query,
            {
                "id": id,
                "input": {"relationship_type": "accesses-to", "toId": marking_id},
            },
        )
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationAdd"]
        )

    def delete_allowed_marking(self, **kwargs) -> Optional[Dict]:
        """Removes access to a marking for a group

        :param id: ID of group to forbid
        :type id: str
        :param marking_id: ID of marking to deny
        :type marking_id: str
        :return: Group after denying access to marking definition
        :rtype: Optional[Dict]
        """
        id = kwargs.get("id", None)
        marking_id = kwargs.get("marking_id", None)

        if id is None or marking_id is None:
            self.opencti.admin_logger.error(
                "[opencti_group] Missing parameters: id and marking_id"
            )
            return None

        self.opencti.admin_logger.info(
            "Forbidding group access to marking definition",
            {"groupId": id, "markingId": marking_id},
        )
        query = (
            """
            mutation GroupEditMarkingRemove($groupId: ID!, $markingId: StixRef!) {
                groupEdit(id: $groupId) {
                    relationDelete(toId: $markingId, relationship_type: "accesses-to") {
                        """
            + self.properties
            + """
                    }
                }
            }
            """
        )
        result = self.opencti.query(query, {"groupId": id, "markingId": marking_id})
        return self.opencti.process_multiple_fields(
            result["data"]["groupEdit"]["relationDelete"]
        )

    def process_multiple_fields(self, data):
        if "roles" in data:
            data["roles"] = self.opencti.process_multiple(data["roles"])
            data["rolesIds"] = self.opencti.process_multiple_ids(data["roles"])
        if "members" in data:
            data["members"] = self.opencti.process_multiple(data["members"])
            data["membersIds"] = self.opencti.process_multiple_ids(data["members"])
        return data



# coding: utf-8

import json
import uuid

from stix2.canonicalization.Canonicalize import canonicalize

from pycti.utils.constants import IdentityTypes


class Identity:
    def __init__(self, opencti):
        self.opencti = opencti
        self.properties = """
            id
            standard_id
            entity_type
            parent_types
            spec_version
            created_at
            updated_at
            status {
                id
                template {
                  id
                  name
                  color
                }
            }
            createdBy {
                ... on Identity {
                    id
                    standard_id
                    entity_type
                    parent_types
                    spec_version
                    identity_class
                    name
                    description
                    roles
                    contact_information
                    x_opencti_aliases
                    x_opencti_reliability
                    created
                    modified
                    objectLabel {
                        id
                        value
                        color
                    }
                }
                ... on Organization {
                    x_opencti_organization_type
                    x_opencti_score
                }
                ... on Individual {
                    x_opencti_firstname
                    x_opencti_lastname
                }
                ... on SecurityPlatform {
                    security_platform_type
                }
            }
            objectMarking {
                id
                standard_id
                entity_type
                definition_type
                definition
                created
                modified
                x_opencti_order
                x_opencti_color
            }
            objectOrganization {
                id
                standard_id
                name
            }
            objectLabel {
                id
                value
                color
            }
            externalReferences {
                edges {
                    node {
                        id
                        standard_id
                        entity_type
                        source_name
                        description
                        url
                        hash
                        external_id
                        created
                        modified
                    }
                }
            }
            revoked
            confidence
            created
            modified
            identity_class
            name
            description
            x_opencti_aliases
            x_opencti_reliability
            contact_information
            ... on Individual {
                x_opencti_firstname
                x_opencti_lastname
            }
            ... on Organization {
                x_opencti_organization_type
                x_opencti_score
            }
            ... on SecurityPlatform {
                security_platform_type
            }
        """
        self.properties_with_files = """
            id
            standard_id
            entity_type
            parent_types
            spec_version
            created_at
            updated_at
            status {
                id
                template {
                  id
                  name
                  color
                }
            }
            createdBy {
                ... on Identity {
                    id
                    standard_id
                    entity_type
                    parent_types
                    spec_version
                    identity_class
                    name
                    description
                    roles
                    contact_information
                    x_opencti_aliases
                    x_opencti_reliability
                    created
                    modified
                    objectLabel {
                        id
                        value
                        color
                    }
                }
                ... on Organization {
                    x_opencti_organization_type
                    x_opencti_score
                }
                ... on Individual {
                    x_opencti_firstname
                    x_opencti_lastname
                }
                ... on SecurityPlatform {
                    security_platform_type
                }
            }
            objectMarking {
                id
                standard_id
                entity_type
                definition_type
                definition
                created
                modified
                x_opencti_order
                x_opencti_color
            }
            objectOrganization {
                id
                standard_id
                name
            }
            objectLabel {
                id
                value
                color
            }
            externalReferences {
                edges {
                    node {
                        id
                        standard_id
                        entity_type
                        source_name
                        description
                        url
                        hash
                        external_id
                        created
                        modified
                        importFiles {
                            edges {
                                node {
                                    id
                                    name
                                    size
                                    metaData {
                                        mimetype
                                        version
                                    }
                                }
                            }
                        }
                    }
                }
            }
            revoked
            confidence
            created
            modified
            identity_class
            name
            description
            x_opencti_aliases
            x_opencti_reliability
            contact_information
            ... on Individual {
                x_opencti_firstname
                x_opencti_lastname
            }
            ... on Organization {
                x_opencti_organization_type
                x_opencti_score
            }
            ... on SecurityPlatform {
                security_platform_type
            }
            importFiles {
                edges {
                    node {
                        id
                        name
                        size
                        metaData {
                            mimetype
                            version
                        }
                    }
                }
            }
        """

    @staticmethod
    def generate_id(name, identity_class):
        data = {"name": name.lower().strip(), "identity_class": identity_class.lower()}
        data = canonicalize(data, utf8=False)
        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
        return "identity--" + id

    @staticmethod
    def generate_id_from_data(data):
        return Identity.generate_id(data["name"], data["identity_class"])

    """
        List Identity objects

        :param types: the list of types
        :param filters: the filters to apply
        :param search: the search keyword
        :param first: return the first n rows from the after ID (or the beginning if not set)
        :param after: ID of the first row for pagination
        :return List of Identity objects
    """

    def list(self, **kwargs):
        types = kwargs.get("types", None)
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        first = kwargs.get("first", 500)
        after = kwargs.get("after", None)
        order_by = kwargs.get("orderBy", None)
        order_mode = kwargs.get("orderMode", None)
        custom_attributes = kwargs.get("customAttributes", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("withPagination", False)
        with_files = kwargs.get("withFiles", False)

        self.opencti.app_logger.info(
            "Listing Identities with filters", {"filters": json.dumps(filters)}
        )
        query = (
            """
            query Identities($types: [String], $filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: IdentitiesOrdering, $orderMode: OrderingMode) {
                identities(types: $types, filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                    edges {
                        node {
                            """
            + (
                custom_attributes
                if custom_attributes is not None
                else (self.properties_with_files if with_files else self.properties)
            )
            + """
                        }
                    }
                    pageInfo {
                        startCursor
                        endCursor
                        hasNextPage
                        hasPreviousPage
                        globalCount
                    }
                }
            }
        """
        )
        result = self.opencti.query(
            query,
            {
                "types": types,
                "filters": filters,
                "search": search,
                "first": first,
                "after": after,
                "orderBy": order_by,
                "orderMode": order_mode,
            },
        )
        if get_all:
            final_data = []
            data = self.opencti.process_multiple(result["data"]["identities"])
            final_data = final_data + data
            while result["data"]["identities"]["pageInfo"]["hasNextPage"]:
                after = result["data"]["identities"]["pageInfo"]["endCursor"]
                self.opencti.app_logger.info("Listing Identities", {"after": after})
                result = self.opencti.query(
                    query,
                    {
                        "filters": filters,
                        "search": search,
                        "first": first,
                        "after": after,
                        "orderBy": order_by,
                        "orderMode": order_mode,
                    },
                )
                data = self.opencti.process_multiple(result["data"]["identities"])
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["identities"], with_pagination
            )

    """
        Read a Identity object

        :param id: the id of the Identity
        :param filters: the filters to apply if no id provided
        :return Identity object
    """

    def read(self, **kwargs):
        id = kwargs.get("id", None)
        filters = kwargs.get("filters", None)
        custom_attributes = kwargs.get("customAttributes", None)
        with_files = kwargs.get("withFiles", False)
        if id is not None:
            self.opencti.app_logger.info("Reading Identity", {"id": id})
            query = (
                """
                query Identity($id: String!) {
                    identity(id: $id) {
                        """
                + (
                    custom_attributes
                    if custom_attributes is not None
                    else (self.properties_with_files if with_files else self.properties)
                )
                + """
                    }
                }
             """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(result["data"]["identity"])
        elif filters is not None:
            result = self.list(filters=filters)
            if len(result) > 0:
                return result[0]
            else:
                return None
        else:
            self.opencti.app_logger.error(
                "[opencti_identity] Missing parameters: id or filters"
            )
            return None

    """
        Create a Identity object

        :param name: the name of the Identity
        :return Identity object
    """

    def create(self, **kwargs):
        type = kwargs.get("type", None)
        stix_id = kwargs.get("stix_id", None)
        created_by = kwargs.get("createdBy", None)
        object_marking = kwargs.get("objectMarking", None)
        object_label = kwargs.get("objectLabel", None)
        external_references = kwargs.get("externalReferences", None)
        revoked = kwargs.get("revoked", None)
        confidence = kwargs.get("confidence", None)
        lang = kwargs.get("lang", None)
        created = kwargs.get("created", None)
        modified = kwargs.get("modified", None)
        name = kwargs.get("name", None)
        description = kwargs.get("description", None)
        contact_information = kwargs.get("contact_information", None)
        roles = kwargs.get("roles", None)
        x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
        security_platform_type = kwargs.get("security_platform_type", None)
        x_opencti_organization_type = kwargs.get("x_opencti_organization_type", None)
        x_opencti_reliability = kwargs.get("x_opencti_reliability", None)
        x_opencti_score = kwargs.get("x_opencti_score", None)
        x_opencti_firstname = kwargs.get("x_opencti_firstname", None)
        x_opencti_lastname = kwargs.get("x_opencti_lastname", None)
        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
        granted_refs = kwargs.get("objectOrganization", None)
        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
        update = kwargs.get("update", False)

        if type is not None and name is not None:
            self.opencti.app_logger.info("Creating Identity", {"name": name})
            input_variables = {
                "stix_id": stix_id,
                "createdBy": created_by,
                "objectMarking": object_marking,
                "objectLabel": object_label,
                "externalReferences": external_references,
                "revoked": revoked,
                "confidence": confidence,
                "lang": lang,
                "created": created,
                "modified": modified,
                "name": name,
                "description": description,
                "contact_information": contact_information,
                "roles": roles,
                "x_opencti_aliases": x_opencti_aliases,
                "x_opencti_stix_ids": x_opencti_stix_ids,
                "x_opencti_workflow_id": x_opencti_workflow_id,
                "update": update,
            }
            if type == IdentityTypes.ORGANIZATION.value:
                query = """
                    mutation OrganizationAdd($input: OrganizationAddInput!) {
                        organizationAdd(input: $input) {
                            id
                            standard_id
                            entity_type
                            parent_types
                        }
                    }
                """
                input_variables["x_opencti_organization_type"] = (
                    x_opencti_organization_type
                )
                input_variables["x_opencti_reliability"] = x_opencti_reliability
                input_variables["x_opencti_score"] = x_opencti_score
                result_data_field = "organizationAdd"
            elif type == IdentityTypes.SECURITYPLATFORM.value:
                query = """
                    mutation SecurityPlatformAdd($input: SecurityPlatformAddInput!) {
                        securityPlatformAdd(input: $input) {
                            id
                            standard_id
                            entity_type
                            parent_types
                        }
                    }
                """
                input_variables["security_platform_type"] = security_platform_type
                # no need for these attributes for security platform
                del input_variables["contact_information"]
                del input_variables["lang"]
                del input_variables["roles"]
                del input_variables["x_opencti_aliases"]
                result_data_field = "securityPlatformAdd"
            elif type == IdentityTypes.INDIVIDUAL.value:
                query = """
                    mutation IndividualAdd($input: IndividualAddInput!) {
                        individualAdd(input: $input) {
                            id
                            standard_id
                            entity_type
                            parent_types
                        }
                    }
                """
                input_variables["objectOrganization"] = granted_refs
                input_variables["x_opencti_firstname"] = x_opencti_firstname
                input_variables["x_opencti_lastname"] = x_opencti_lastname
                input_variables["x_opencti_reliability"] = x_opencti_reliability
                result_data_field = "individualAdd"
            elif type == IdentityTypes.SYSTEM.value:
                query = """
                    mutation SystemAdd($input: SystemAddInput!) {
                        systemAdd(input: $input) {
                            id
                            standard_id
                            entity_type
                            parent_types
                        }
                    }
                """
                input_variables["objectOrganization"] = granted_refs
                input_variables["x_opencti_firstname"] = x_opencti_firstname
                input_variables["x_opencti_lastname"] = x_opencti_lastname
                input_variables["x_opencti_reliability"] = x_opencti_reliability
                result_data_field = "systemAdd"
            else:
                query = """
                    mutation IdentityAdd($input: IdentityAddInput!) {
                        identityAdd(input: $input) {
                            id
                            standard_id
                            entity_type
                            parent_types
                        }
                    }
                """
                input_variables["type"] = type
                result_data_field = "identityAdd"
            result = self.opencti.query(
                query,
                {
                    "input": input_variables,
                },
            )
            return self.opencti.process_multiple_fields(
                result["data"][result_data_field]
            )
        else:
            self.opencti.app_logger.error(
                "Missing parameters: type, name and description"
            )

    """
        Import an Identity object from a STIX2 object

        :param stixObject: the Stix-Object Identity
        :return Identity object
    """

    def import_from_stix2(self, **kwargs):
        stix_object = kwargs.get("stixObject", None)
        extras = kwargs.get("extras", {})
        update = kwargs.get("update", False)
        if stix_object is not None:
            type = "Organization"
            if "identity_class" in stix_object:
                if stix_object["identity_class"] == "individual":
                    type = "Individual"
                elif stix_object["identity_class"] == "class":
                    type = "Sector"
                elif stix_object["identity_class"] == "system":
                    type = "System"
                elif stix_object["identity_class"] == "securityplatform":
                    type = "SecurityPlatform"

            # Search in extensions
            if "x_opencti_aliases" not in stix_object:
                stix_object["x_opencti_aliases"] = (
                    self.opencti.get_attribute_in_extension("aliases", stix_object)
                )
            if "x_opencti_organization_type" not in stix_object:
                stix_object["x_opencti_organization_type"] = (
                    self.opencti.get_attribute_in_extension(
                        "organization_type", stix_object
                    )
                )
            if "security_platform_type" not in stix_object:
                stix_object["security_platform_type"] = (
                    self.opencti.get_attribute_in_extension(
                        "security_platform_type", stix_object
                    )
                )
            if "x_opencti_reliability" not in stix_object:
                stix_object["x_opencti_reliability"] = (
                    self.opencti.get_attribute_in_extension("reliability", stix_object)
                )
            if "x_opencti_score" not in stix_object:
                stix_object["x_opencti_score"] = (
                    self.opencti.get_attribute_in_extension("score", stix_object)
                )
            if "x_opencti_organization_type" not in stix_object:
                stix_object["x_opencti_organization_type"] = (
                    self.opencti.get_attribute_in_extension(
                        "organization_type", stix_object
                    )
                )
            if "x_opencti_firstname" not in stix_object:
                stix_object["x_opencti_firstname"] = (
                    self.opencti.get_attribute_in_extension("firstname", stix_object)
                )
            if "x_opencti_lastname" not in stix_object:
                stix_object["x_opencti_lastname"] = (
                    self.opencti.get_attribute_in_extension("lastname", stix_object)
                )
            if "x_opencti_stix_ids" not in stix_object:
                stix_object["x_opencti_stix_ids"] = (
                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
                )
            if "x_opencti_granted_refs" not in stix_object:
                stix_object["x_opencti_granted_refs"] = (
                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
                )
            if "x_opencti_workflow_id" not in stix_object:
                stix_object["x_opencti_workflow_id"] = (
                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                )

            return self.create(
                type=type,
                stix_id=stix_object["id"],
                createdBy=(
                    extras["created_by_id"] if "created_by_id" in extras else None
                ),
                objectMarking=(
                    extras["object_marking_ids"]
                    if "object_marking_ids" in extras
                    else None
                ),
                objectLabel=(
                    extras["object_label_ids"] if "object_label_ids" in extras else None
                ),
                externalReferences=(
                    extras["external_references_ids"]
                    if "external_references_ids" in extras
                    else None
                ),
                revoked=stix_object["revoked"] if "revoked" in stix_object else None,
                confidence=(
                    stix_object["confidence"] if "confidence" in stix_object else None
                ),
                lang=stix_object["lang"] if "lang" in stix_object else None,
                created=stix_object["created"] if "created" in stix_object else None,
                modified=stix_object["modified"] if "modified" in stix_object else None,
                name=stix_object["name"],
                description=(
                    self.opencti.stix2.convert_markdown(stix_object["description"])
                    if "description" in stix_object
                    else None
                ),
                contact_information=(
                    self.opencti.stix2.convert_markdown(
                        stix_object["contact_information"]
                    )
                    if "contact_information" in stix_object
                    else None
                ),
                roles=stix_object["roles"] if "roles" in stix_object else None,
                x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
                x_opencti_organization_type=(
                    stix_object["x_opencti_organization_type"]
                    if "x_opencti_organization_type" in stix_object
                    else None
                ),
                security_platform_type=(
                    stix_object["security_platform_type"]
                    if "security_platform_type" in stix_object
                    else None
                ),
                x_opencti_reliability=(
                    stix_object["x_opencti_reliability"]
                    if "x_opencti_reliability" in stix_object
                    else None
                ),
                x_opencti_score=(
                    stix_object["x_opencti_score"]
                    if "x_opencti_score" in stix_object
                    else None
                ),
                x_opencti_firstname=(
                    stix_object["x_opencti_firstname"]
                    if "x_opencti_firstname" in stix_object
                    else None
                ),
                x_opencti_lastname=(
                    stix_object["x_opencti_lastname"]
                    if "x_opencti_lastname" in stix_object
                    else None
                ),
                x_opencti_stix_ids=(
                    stix_object["x_opencti_stix_ids"]
                    if "x_opencti_stix_ids" in stix_object
                    else None
                ),
                objectOrganization=(
                    stix_object["x_opencti_granted_refs"]
                    if "x_opencti_granted_refs" in stix_object
                    else None
                ),
                x_opencti_workflow_id=(
                    stix_object["x_opencti_workflow_id"]
                    if "x_opencti_workflow_id" in stix_object
                    else None
                ),
                update=update,
            )
        else:
            self.opencti.app_logger.error(
                "[opencti_identity] Missing parameters: stixObject"
            )



class OpenCTIApiTrash:
    """OpenCTIApiTrash"""

    def __init__(self, api):
        self.api = api

    def restore(self, operation_id: str):
        query = """
            mutation DeleteOperationRestore($id: ID!) {
                deleteOperationRestore(id: $id)
            }
           """
        self.api.query(
            query,
            {
                "id": operation_id,
            },
        )

    def delete(self, **kwargs):
        """Delete a trash item given its ID

        :param id: ID for the delete operation on the platform.
        :type id: str
        """
        id = kwargs.get("id", None)
        if id is None:
            self.api.admin_logger.error(
                "[opencti_trash] Cant confirm delete, missing parameter: id"
            )
            return None
        query = """
            mutation DeleteOperationConfirm($id: ID!) {
                deleteOperationConfirm(id: $id)
            }
        """
        self.api.query(
            query,
            {
                "id": id,
            },
        )



class OpenCTIApiWorkspace:
    """OpenCTIApiWorkspace"""

    def __init__(self, api):
        self.api = api

    def delete(self, **kwargs):
        id = kwargs.get("id", None)
        if id is None:
            self.api.admin_logger.error(
                "[opencti_workspace] Cant delete workspace, missing parameter: id"
            )
            return None
        query = """
            mutation WorkspaceDelete($id: ID!) {
                workspaceDelete(id: $id)
            }
           """
        self.api.query(
            query,
            {
                "id": id,
            },
        )



# -*- coding: utf-8 -*-
__version__ = "6.7.6"

from .api.opencti_api_client import OpenCTIApiClient
from .api.opencti_api_connector import OpenCTIApiConnector
from .api.opencti_api_work import OpenCTIApiWork
from .connector.opencti_connector import ConnectorType, OpenCTIConnector
from .connector.opencti_connector_helper import (
    OpenCTIConnectorHelper,
    get_config_variable,
)
from .connector.opencti_metric_handler import OpenCTIMetricHandler
from .entities.opencti_attack_pattern import AttackPattern
from .entities.opencti_campaign import Campaign

# Administrative entities
from .entities.opencti_capability import Capability
from .entities.opencti_case_incident import CaseIncident
from .entities.opencti_case_rfi import CaseRfi
from .entities.opencti_case_rft import CaseRft
from .entities.opencti_channel import Channel
from .entities.opencti_course_of_action import CourseOfAction
from .entities.opencti_data_component import DataComponent
from .entities.opencti_data_source import DataSource
from .entities.opencti_external_reference import ExternalReference
from .entities.opencti_feedback import Feedback
from .entities.opencti_group import Group
from .entities.opencti_grouping import Grouping
from .entities.opencti_identity import Identity
from .entities.opencti_incident import Incident
from .entities.opencti_indicator import Indicator
from .entities.opencti_infrastructure import Infrastructure
from .entities.opencti_intrusion_set import IntrusionSet
from .entities.opencti_kill_chain_phase import KillChainPhase
from .entities.opencti_label import Label
from .entities.opencti_location import Location
from .entities.opencti_malware import Malware
from .entities.opencti_malware_analysis import MalwareAnalysis
from .entities.opencti_marking_definition import MarkingDefinition
from .entities.opencti_note import Note
from .entities.opencti_observed_data import ObservedData
from .entities.opencti_opinion import Opinion
from .entities.opencti_report import Report
from .entities.opencti_role import Role
from .entities.opencti_settings import Settings
from .entities.opencti_stix_core_relationship import StixCoreRelationship
from .entities.opencti_stix_cyber_observable import StixCyberObservable
from .entities.opencti_stix_domain_object import StixDomainObject
from .entities.opencti_stix_nested_ref_relationship import StixNestedRefRelationship
from .entities.opencti_stix_object_or_stix_relationship import (
    StixObjectOrStixRelationship,
)
from .entities.opencti_stix_sighting_relationship import StixSightingRelationship
from .entities.opencti_task import Task
from .entities.opencti_threat_actor import ThreatActor
from .entities.opencti_threat_actor_group import ThreatActorGroup
from .entities.opencti_threat_actor_individual import ThreatActorIndividual
from .entities.opencti_tool import Tool
from .entities.opencti_user import User
from .entities.opencti_vulnerability import Vulnerability
from .utils.constants import (
    CustomObjectCaseIncident,
    CustomObjectChannel,
    CustomObjectTask,
    CustomObservableBankAccount,
    CustomObservableCredential,
    CustomObservableCryptocurrencyWallet,
    CustomObservableCryptographicKey,
    CustomObservableHostname,
    CustomObservableMediaContent,
    CustomObservablePaymentCard,
    CustomObservablePersona,
    CustomObservablePhoneNumber,
    CustomObservableText,
    CustomObservableTrackingNumber,
    CustomObservableUserAgent,
    MultipleRefRelationship,
    StixCyberObservableTypes,
    StixMetaTypes,
)
from .utils.opencti_stix2 import (
    STIX_EXT_MITRE,
    STIX_EXT_OCTI,
    STIX_EXT_OCTI_SCO,
    OpenCTIStix2,
)
from .utils.opencti_stix2_splitter import OpenCTIStix2Splitter
from .utils.opencti_stix2_update import OpenCTIStix2Update
from .utils.opencti_stix2_utils import OpenCTIStix2Utils

__all__ = [
    "AttackPattern",
    "Campaign",
    "CaseIncident",
    "CaseRfi",
    "CaseRft",
    "Channel",
    "Task",
    "ConnectorType",
    "CourseOfAction",
    "DataComponent",
    "DataSource",
    "ExternalReference",
    "Feedback",
    "Grouping",
    "Identity",
    "Incident",
    "Indicator",
    "Infrastructure",
    "IntrusionSet",
    "KillChainPhase",
    "Label",
    "Location",
    "Malware",
    "MalwareAnalysis",
    "MarkingDefinition",
    "Note",
    "ObservedData",
    "OpenCTIApiClient",
    "OpenCTIApiConnector",
    "OpenCTIApiWork",
    "OpenCTIConnector",
    "OpenCTIConnectorHelper",
    "OpenCTIMetricHandler",
    "OpenCTIStix2",
    "OpenCTIStix2Splitter",
    "OpenCTIStix2Update",
    "OpenCTIStix2Utils",
    "Opinion",
    "Report",
    "StixCoreRelationship",
    "StixCyberObservable",
    "StixNestedRefRelationship",
    "StixCyberObservableTypes",
    "StixDomainObject",
    "StixMetaTypes",
    "MultipleRefRelationship",
    "StixObjectOrStixRelationship",
    "StixSightingRelationship",
    "ThreatActor",
    "ThreatActorGroup",
    "ThreatActorIndividual",
    "Tool",
    "Vulnerability",
    "get_config_variable",
    "CustomObjectCaseIncident",
    "CustomObjectTask",
    "CustomObjectChannel",
    "StixCyberObservableTypes",
    "CustomObservableCredential",
    "CustomObservableHostname",
    "CustomObservableUserAgent",
    "CustomObservableBankAccount",
    "CustomObservableCryptographicKey",
    "CustomObservableCryptocurrencyWallet",
    "CustomObservablePaymentCard",
    "CustomObservablePersona",
    "CustomObservablePhoneNumber",
    "CustomObservableTrackingNumber",
    "CustomObservableText",
    "CustomObservableMediaContent",
    "STIX_EXT_MITRE",
    "STIX_EXT_OCTI_SCO",
    "STIX_EXT_OCTI",
    "Capability",
    "Role",
    "Group",
    "User",
    "Settings",
]



class OpenCTIApiDraft:
    """OpenCTIApiDraft"""

    def __init__(self, api):
        self.api = api

    def delete(self, **kwargs):
        id = kwargs.get("id", None)
        query = """
            mutation DraftWorkspaceDelete($id: ID!) {
                draftWorkspaceDelete(id: $id)
            }
           """
        self.api.query(
            query,
            {
                "id": id,
            },
        )



class Stix:
    def __init__(self, opencti):
        self.opencti = opencti

    """
        Delete a Stix element

        :param id: the Stix element id
        :return void
    """

    def delete(self, **kwargs):
        id = kwargs.get("id", None)
        force_delete = kwargs.get("force_delete", True)
        if id is not None:
            self.opencti.app_logger.info("Deleting Stix element", {"id": id})
            query = """
                 mutation StixEdit($id: ID!, $forceDelete: Boolean) {
                     stixEdit(id: $id) {
                         delete(forceDelete: $forceDelete)
                     }
                 }
             """
            self.opencti.query(query, {"id": id, "forceDelete": force_delete})
        else:
            self.opencti.app_logger.error("[opencti_stix] Missing parameters: id")
            return None

    """
            Merge a Stix-Object object field
    
            :param id: the Stix-Object id
            :param key: the key of the field
            :param value: the value of the field
            :return The updated Stix-Object object
        """

    def merge(self, **kwargs):
        id = kwargs.get("id")
        stix_objects_ids = kwargs.get("object_ids")
        if id is not None and stix_objects_ids is not None:
            self.opencti.app_logger.info(
                "Merging Stix object", {"id": id, "sources": ",".join(stix_objects_ids)}
            )
            query = """
                        mutation StixEdit($id: ID!, $stixObjectsIds: [String]!) {
                            stixEdit(id: $id) {
                                merge(stixObjectsIds: $stixObjectsIds) {
                                    id
                                    standard_id
                                    entity_type
                                }
                            }
                        }
                    """
            result = self.opencti.query(
                query,
                {
                    "id": id,
                    "stixObjectsIds": stix_objects_ids,
                },
            )
            return self.opencti.process_multiple_fields(
                result["data"]["stixEdit"]["merge"]
            )
        else:
            self.opencti.app_logger.error(
                "[opencti_stix] Missing parameters: id and object_ids"
            )
            return None



class OpenCTIApiPublicDashboard:
    """OpenCTIApiPublicDashboard"""

    def __init__(self, api):
        self.api = api

    def delete(self, **kwargs):
        id = kwargs.get("id", None)
        if id is not None:
            query = """
                mutation PublicDashboardDelete($id: ID!) {
                    publicDashboardDelete(id: $id)
                }
               """
            self.api.query(
                query,
                {
                    "id": id,
                },
            )
        else:
            self.opencti.app_logger.error(
                "[stix_public_dashboard] Cant delete public dashboard, missing parameters: id"
            )
            return None



class OpenCTIApiNotification:
    """OpenCTIApiJob"""

    def __init__(self, api):
        self.api = api

    def delete(self, **kwargs):
        notification_id = kwargs.get("id", None)
        self.api.app_logger.info(
            "Deleting notifcation", {"notification_id": notification_id}
        )
        query = """
            mutation notificationDelete($id: ID!) {
                notificationDelete(id: $id)
            }
           """
        self.api.query(query, {"id": notification_id})

    def update_field(self, **kwargs):
        notification_id = kwargs.get("id", None)
        input = kwargs.get("input", None)
        for input_value in input:
            if input_value["key"] == "is_read":
                is_read_value = bool(input_value["value"][0])
                self.mark_as_read(notification_id, is_read_value)

    def mark_as_read(self, notification_id: str, read: bool):
        self.api.app_logger.info(
            "Marking notifcation as read",
            {"notification_id": notification_id, "read": read},
        )
        query = """
                mutation notificationMarkRead($id: ID!, $read: Boolean!) {
                    notificationMarkRead(id: $id, read: $read) {
                        id
                    }
                }
               """
        self.api.query(query, {"id": notification_id, "read": read})
